@@ -4,68 +4,27 @@ local tools = {}
 
 local fs = require("luarocks.fs")
 local dir = require("luarocks.dir")
-local cfg = require("luarocks.core.cfg")
+local cfg = require("luarocks.cfg")
 
-local vars = setmetatable({}, { __index = function(_,k) return cfg.variables[k] end })
+local vars = cfg.variables
 
 local dir_stack = {}
 
-do
-   local tool_cache = {}
-   
-   local tool_options = {
-      downloader = {
-         desc = "downloader",
-         { var = "WGET", name = "wget" },
-         { var = "CURL", name = "curl" },
-      },
-      md5checker = {
-         desc = "MD5 checker",
-         { var = "MD5SUM", name = "md5sum" },
-         { var = "OPENSSL", name = "openssl", cmdarg = "md5", checkarg = "version" },
-         { var = "MD5", name = "md5", checkarg = "-shello" },
-      },
-   }
-
-   function tools.which_tool(tooltype)
-      local tool = tool_cache[tooltype]
-      local names = {}
-      if not tool then
-         for _, opt in ipairs(tool_options[tooltype]) do
-            table.insert(names, opt.name)
-            if fs.is_tool_available(vars[opt.var], opt.name, opt.checkarg) then
-               tool = opt
-               tool_cache[tooltype] = opt
-               break
-            end
-         end
-      end
-      if not tool then
-         local tool_names = table.concat(names, ", ", 1, #names - 1) .. " or " .. names[#names]
-         return nil, "no " .. tool_options[tooltype].desc .. " tool available," .. " please install " .. tool_names .. " in your system"
-      end
-      return tool.name, vars[tool.var] .. (tool.cmdarg and " "..tool.cmdarg or "")
+--- Obtain current directory.
+-- Uses the module's internal directory stack.
+-- @return string: the absolute pathname of the current directory.
+function tools.current_dir()
+   local current = cfg.cache_pwd
+   if not current then
+      local pipe = io.popen(fs.quiet_stderr(fs.Q(vars.PWD)))
+      current = pipe:read("*l")
+      pipe:close()
+      cfg.cache_pwd = current
    end
-end
-
-do
-   local cache_pwd
-   --- Obtain current directory.
-   -- Uses the module's internal directory stack.
-   -- @return string: the absolute pathname of the current directory.
-   function tools.current_dir()
-      local current = cache_pwd
-      if not current then
-         local pipe = io.popen(fs.quiet_stderr(vars.PWD))
-         current = pipe:read("*l")
-         pipe:close()
-         cache_pwd = current
-      end
-      for _, directory in ipairs(dir_stack) do
-         current = fs.absolute_name(directory, current)
-      end
-      return current
+   for _, directory in ipairs(dir_stack) do
+      current = fs.absolute_name(directory, current)
    end
+   return current
 end
 
 --- Change the current directory.
@@ -87,12 +46,7 @@ end
 -- Allows leaving a directory (e.g. for deleting it) in
 -- a crossplatform way.
 function tools.change_dir_to_root()
-   local curr_dir = fs.current_dir()
-   if not curr_dir or not fs.is_dir(curr_dir) then
-      return false
-   end
    table.insert(dir_stack, "/")
-   return true
 end
 
 --- Change working directory to the previous in the directory stack.
@@ -123,7 +77,7 @@ end
 -- @param at string: directory to list
 -- @return nil
 function tools.dir_iterator(at)
-   local pipe = io.popen(fs.command_at(at, vars.LS, true))
+   local pipe = io.popen(fs.command_at(at, fs.Q(vars.LS)))
    for file in pipe:lines() do
       if file ~= "." and file ~= ".." then
          coroutine.yield(file)
@@ -138,8 +92,6 @@ end
 -- resulting local filename of the remote file as the basename of the URL;
 -- if that is not correct (due to a redirection, for example), the local
 -- filename can be given explicitly as this second argument.
--- @param cache boolean: compare remote timestamps via HTTP HEAD prior to
--- re-downloading the file.
 -- @return (boolean, string): true and the filename on success,
 -- false and the error message on failure.
 function tools.use_downloader(url, filename, cache)
@@ -147,22 +99,16 @@ function tools.use_downloader(url, filename, cache)
    assert(type(filename) == "string" or not filename)
 
    filename = fs.absolute_name(filename or dir.base_name(url))
-   
-   local downloader, err = fs.which_tool("downloader")
-   if not downloader then
-      return nil, err
-   end
 
-   local ok = false
-   if downloader == "wget" then
-      local wget_cmd = vars.WGET.." "..vars.WGETNOCERTFLAG.." --no-cache --user-agent=\""..cfg.user_agent.." via wget\" --quiet "
+   local ok
+   if cfg.downloader == "wget" then
+      local wget_cmd = fs.Q(vars.WGET).." "..vars.WGETNOCERTFLAG.." --no-cache --user-agent=\""..cfg.user_agent.." via wget\" --quiet "
       if cfg.connection_timeout and cfg.connection_timeout > 0 then
-        wget_cmd = wget_cmd .. "--timeout="..tostring(cfg.connection_timeout).." --tries=1 "
+        wget_cmd = wget_cmd .. "--timeout="..tonumber(cfg.connection_timeout).." --tries=1 "
       end
       if cache then
          -- --timestamping is incompatible with --output-document,
          -- but that's not a problem for our use cases.
-         fs.delete(filename .. ".unixtime")
          fs.change_dir(dir.dir_name(filename))
          ok = fs.execute_quiet(wget_cmd.." --timestamping ", url)
          fs.pop_dir()
@@ -171,44 +117,40 @@ function tools.use_downloader(url, filename, cache)
       else
          ok = fs.execute_quiet(wget_cmd, url)
       end
-   elseif downloader == "curl" then
-      local curl_cmd = vars.CURL.." "..vars.CURLNOCERTFLAG.." -f -L --user-agent \""..cfg.user_agent.." via curl\" "
+   elseif cfg.downloader == "curl" then
+      local curl_cmd = fs.Q(vars.CURL).." "..vars.CURLNOCERTFLAG.." -f -L --user-agent \""..cfg.user_agent.." via curl\" "
       if cfg.connection_timeout and cfg.connection_timeout > 0 then
-        curl_cmd = curl_cmd .. "--connect-timeout "..tostring(cfg.connection_timeout).." "
+        curl_cmd = curl_cmd .. "--connect-timeout "..tonumber(cfg.connection_timeout).." "
       end
-      if cache then
-         curl_cmd = curl_cmd .. " -R -z \"" .. filename .. "\" "
-      end
-      ok = fs.execute_string(fs.quiet_stderr(curl_cmd..fs.Q(url).." --output "..fs.Q(filename)))
+      ok = fs.execute_string(fs.quiet_stderr(curl_cmd..fs.Q(url).." > "..fs.Q(filename)))
    end
    if ok then
       return true, filename
    else
-      os.remove(filename)
-      return false, "failed downloading " .. url
+      return false
    end
 end
 
+local md5_cmd = {
+   md5sum = fs.Q(vars.MD5SUM),
+   openssl = fs.Q(vars.OPENSSL).." md5",
+   md5 = fs.Q(vars.MD5),
+}
+
 --- Get the MD5 checksum for a file.
 -- @param file string: The file to be computed.
 -- @return string: The MD5 checksum or nil + message
 function tools.get_md5(file)
-   local ok, md5checker = fs.which_tool("md5checker")
-   if not ok then
-      return false, md5checker
-   end
-
-   local pipe = io.popen(md5checker.." "..fs.Q(fs.absolute_name(file)))
-   local computed = pipe:read("*l")
+   local cmd = md5_cmd[cfg.md5checker]
+   if not cmd then return nil, "no MD5 checker command configured" end
+   local pipe = io.popen(cmd.." "..fs.Q(fs.absolute_name(file)))
+   local computed = pipe:read("*a")
    pipe:close()
    if computed then
       computed = computed:match("("..("%x"):rep(32)..")")
    end
-   if computed then
-      return computed
-   else
-      return nil, "Failed to compute MD5 hash for file "..tostring(fs.absolute_name(file))
-   end
+   if computed then return computed end
+   return nil, "Failed to compute MD5 hash for file "..tostring(fs.absolute_name(file))
 end
 
 return tools
